package geneticWedge.gp;

/**An EvaluativeStoppingCriterion checks some property of the fitness function to decide whether evolution should be stopped.
This may be contrasted with FixedGenerationStoppingCriterion which does not implememt early stopping.*/
public abstract class EvaluativeStoppingCriterion extends StoppingCriterion
{
	boolean maximiseValue=false; //if true, evolution should increase the fitness function
	boolean useValidationData;
	boolean useTrainingData;//added 211107. allows combination of training and validation fitnesses

	/**If true, it is assumed that a high-valued FitnessFunction is desirable. Therefore evolution would usually be stopped
	if the FitnessFunction is no longer increasing. On the other hand, if set to false evolution should be stopped when
	the FitnessFunction is no longer decreasing.*/
	public void setMaximise(boolean maximiseValue)
	{
		this.maximiseValue=maximiseValue;
	}

	/**Early Stopping may be carried out based on changes in the training fitness or the validation fitness. This
	function returns true if early stopping is assessed against validation data, false if assessed against training data.*/
	public boolean usesValidation()
	{
		return useValidationData;
	}
}
