package geneticWedge.gp;

/**A FitnessStoppingCriterion stops evolution based on some property of the FitnessFunction.*/
public class FitnessStoppingCriterion extends EvaluativeStoppingCriterion
{
	private double targetVal;
	private boolean averageValues=true;
	//private boolean maximiseValue=false; //if true stop once the value has been exceeded, if false stop once the value has been undercut
	//private boolean useValidationData;

	/**Default constructor
	@param fitnessValue The target fitness. Once this value is reached evolution will be stopped.
	@param averageValues If true, the average value of the fitness across the whole population will be compared
	against the target value. If false, the value of the best member of the population will be used.
	@param useValidationData If true, fitness values on the validation data will be used to assess whether
	a stop should occur. If false, training data will be used.*/
	public FitnessStoppingCriterion(double fitnessValue, boolean averageValues, boolean useValidationData)
	{
		targetVal=fitnessValue;
		this.averageValues=averageValues;
		this.useValidationData=useValidationData;
	}

	/**Will return true only if the stop criterion has been reached.*/
	public boolean stopReached(AbstractPopulation pop)
	{
		if(maximiseValue)
		{
			if(averageValues)
			{
				if(useValidationData)
				{
					if(pop.getAverageValidationFitness()>=targetVal)
						return true;
					else
						return false;
				}
				else
				{
					if(pop.getAverageTrainingFitness()>=targetVal)
						return true;
					else
						return false;
				}
			}
			else
			{
				if(useValidationData)
				{
					if(pop.getBestValidationFitness()>=targetVal)
						return true;
					else
						return false;
				}
				else
				{
					if(pop.getBestTrainingFitness()>=targetVal)
						return true;
					else
						return false;
				}
			}
		}
		else
		{
			if(averageValues)
			{
				if(useValidationData)
				{
					if(pop.getAverageValidationFitness()<=targetVal)
						return true;
					else
						return false;
				}
				else
				{
					if(pop.getAverageTrainingFitness()<=targetVal)
						return true;
					else
						return false;
				}
			}
			else
			{
				if(useValidationData)
				{
					if(pop.getBestValidationFitness()<=targetVal)
						return true;
					else
						return false;
				}
				else
				{
					if(pop.getBestTrainingFitness()<=targetVal)
						return true;
					else
						return false;
				}
			}
		}
	}

	/**Returns a string representation of this FitnessStoppingCriterion. This is useful for generating results files
	containing information about a GP run.*/
	public String toString()
	{
		String str="FitnessStoppingCriterion. ";
		if(averageValues)
			str+="Average";
		else
			str+="Best";

		if(useValidationData)
			str+=" validation value";
		else
			str+=" training value";

		if(maximiseValue)
			str+=" >= "+targetVal+".";
		else
			str+=" <= "+targetVal+".";

		return str;
	}
}