package geneticWedge.gp;

/**If a FixedGenerationsStoppingCriterion is used, early stopping will not occur.*/
public class FixedGenerationsStoppingCriterion extends StoppingCriterion
{
	private int noOfGenerations;

	/**Default constructor sets the number of generations for which to evolve. A generation is considered to have passed
	when the number of new offspring created equals the inital population size.*/
	public FixedGenerationsStoppingCriterion(int noOfGenerations)
	{
		this.noOfGenerations=noOfGenerations;
	}

	/**Returns true if the required number of generations of evolution have occurred.*/
	public boolean stopReached(AbstractPopulation pop)
	{
		if(pop.getGenerationNumber()>=noOfGenerations)
			return true;
		else
			return false;
	}

	/**Returns a String representation of this FixedGenerationsStopping Criterion. This is useful for generating results files
	containing information about a GP run.*/
	public String toString()
	{
		String str="FixedGenerationsStoppingCriterion. "+noOfGenerations+" generations.";
		return str;
	}
}