/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package geneticWedge.gp;

import java.util.*;
import geneticWedge.gp.events.*;
import java.io.*;

/**
Extends Population to include the calls to the GPDone & GPGeneration events
in the evolve method, and addition of supporting methods.
For use when the GP need to run Interactively - e.g. JavaGP GUI.
@author mcdssso
 */
public class PopulationV extends Population {

    Vector<GPEventHandler> GPDoneHandlers = new Vector<GPEventHandler>();
    Vector<GPEventHandler> GPGenerationHandlers = new Vector<GPEventHandler>();
    private volatile boolean stopnowflag = false;

    public void stopnow() {
        stopnowflag = true;
    }

    public void addGPDoneHandler(GPEventHandler gh) {
        if (gh != null) {
            GPDoneHandlers.add(gh);
        }
    }

    public void addGPGenerationHandler(GPEventHandler gh) {
        if (gh != null) {
            GPGenerationHandlers.add(gh);
        }
    }

    public void removeGPDoneHandler(GPEventHandler gh) {
        if (gh != null) {
            GPDoneHandlers.remove(gh);
        }
    }

    public void removeGPGeneration(GPEventHandler gh) {
        if (gh != null) {
            GPGenerationHandlers.remove(gh);
        }
    }

    void fireGPDoneEvent() {
        GPDoneEvent ev = new GPDoneEvent(this);
        Enumeration en = GPDoneHandlers.elements();
        while (en.hasMoreElements()) {
            GPEventHandler gh = (GPEventHandler) en.nextElement();
            gh.GPDone(ev);
        }
    }

    void fireGPGenerationEvent(GPGenerationEvent ev) {
        Enumeration en = GPGenerationHandlers.elements();
        while (en.hasMoreElements()) {
            GPEventHandler gh = (GPEventHandler) en.nextElement();
            gh.GPGeneration(ev);
        }
    }

    protected void fireGen() {
        GPGenerationEvent ev = new GPGenerationEvent(this);
        ev.setBestIndividual(this.getBestIndividual());
        ev.setBestTrainingFitness(this.bestTrainFitness);
        ev.setBestValidationfitness(this.bestValidationFitness);
        ev.setGenerationNumber(this.generationNumber);
        this.fireGPGenerationEvent(ev);
    }

    /**The Population constructor takes the population size and a PopulationDescriptor as variables.*/
    public PopulationV(int populationSize, PopulationDescriptor descriptor) {
        super(populationSize, descriptor);
    }

    /**If this method is called in steadyState mode with a selectMethod that uses roulette wheel selection
    (FITNESS_PROPORTIONAL, RANK or FUSS), elitist selection will not occur, even if an elite size has been set.
    In order to acieve elitist selection with these methods, generational evolution must be applied.*/
    @Override
    public boolean evolve(boolean steadyState, SelectionMethod selectMethod, double[][] inputs, double[] targetOutputs, double[][] validationInputs, double[] validationTargetOutputs, StoppingCriterion stopCriterion) {
        evolutionNumber = 0;
        stopnowflag = false;
        if (printInfo) {
            System.out.println("Population evolving.");
        }
        try {
            if (crossover.getBroodUse()) {
                crossover.setData(inputs, targetOutputs);
                crossover.setFitnessFunction(fitnessFunction);
            }
            if (printInfo) {
                System.out.println("Setting fitness vals");
            }
            setFitnessValues(inputs, targetOutputs);
            if (printInfo) {
                System.out.println("Finished setting fitness vals");
            }

            int selPressure = popDescriptor.getInitialSelectionPressure();
            if (selPressure > 1) {
                setFitnessValues(inputs, targetOutputs);
                Individual[] orderedIndividuals = (Individual[]) (ShellSortIndividuals()[0]);

                popSize /= selPressure;
                startingPopSize /= selPressure;
                individuals = new Individual[popSize];
                for (int i = 0; i < popSize; i++) {
                    individuals[i] = orderedIndividuals[i];
                }
            }
            if (stopCriterion instanceof EvaluativeStoppingCriterion) {
                switch (fitnessFunction) {
                    case MSE:
                    case RMSE:
                    case ABSOLUTE_ERROR:
                    case AOREC:
                    case COMBINED_CLASSIFICATION_ERROR:
                    case CLUSTERED_REGRESSION:
                        ((EvaluativeStoppingCriterion) stopCriterion).setMaximise(false);
                        break;
                    case NUMBER_CORRECT_BINARY:
                    case AUROC:
                    case SENSITIVITY:
                    case SPECIFICITY:
                    case PPV:
                    case NPV:
                        ((EvaluativeStoppingCriterion) stopCriterion).setMaximise(true);
                        break;
                }
                if (printInfo) {
                    System.out.println("Setting validation fitness vals");
                }
                if (((EvaluativeStoppingCriterion) stopCriterion).usesValidation()) {
                    setValidationFitnessValues(validationInputs, validationTargetOutputs);
                }
                if (printInfo) {
                    System.out.println("Finished setting validation fitness vals");
                }
                if (stopCriterion instanceof StagnationStoppingCriterion) {
                    super.savePopulation();
                }
            }
            runtime.gc();
            boolean distanceSet = false;
            if (performanceFunctions != null) {
                if (printInfo) {
                    System.out.println("Setting performance functions");
                }
                for (int i = 0; i < performanceFunctions.length; i++) {
                    switch (performanceFunctions[i]) {
                        case DIVERSITY_DISTANCE:
                            setAllDistances();
                            distanceSet = true;
                            break;
                        case FUNCTION_COMPLEXITY:
                            setAllComplexities();
                            break;
                        case VARIABLE_MAX:
                        case VARIABLE_MIN:
                            setAllNumberOfInputsUsed();
                            break;
                    }
                }
                if (fitnessShare != null && shareType.equals(FitnessShare.ShareType.GENE_DISTANCE) && !distanceSet) {
                    setAllDistances();
                    distanceSet = true;
                }
                if (printInfo) {
                    System.out.println("Finished setting performance functions");
                }
                if (isMultiObjective) {
                    if (printInfo) {
                        System.out.println("Removing dominated and duplicated individuals");
                    }
                    if (!keepDominatedIndividuals) {
                        super.removeDuplicatedIndividuals();
                    }
                    setDominatedMatrix();
                    if (!keepDominatedIndividuals) {
                        super.removeDominatedIndividuals();
                        //setFitnessValues(inputs, targetOutputs);//shouldn't be necessary
                    }
                    if (printInfo) {
                        System.out.println("Finished removing dominated and duplicated individuals");
                    }
                }
            }
            if (fitnessShare != null && shareType.equals(FitnessShare.ShareType.GENE_DISTANCE) && !distanceSet) {
                setAllDistances();
            }
            if (printInfo) {
                System.out.println("Generation " + generationNumber + ", Mean Fitness= " + averageTrainFitness + ". Best fitness=" + bestTrainFitness);
                System.out.println("Mean val Fitness= " + averageValidationFitness + ". Best val fitness=" + bestValidationFitness);
                System.out.println("average length=" + getAverageIndividualLength()/*+", diversity="+getDiversity()*/);//don't check diversity - slows down prog
            }
            //if(stopCriterion.stopIfNoDiversity()&&getDiversity()==0.0)
            if (stopCriterion.stopIfNoDiversity() && hasNoDiversity()) {
                return true;
            }
            if (collectTransmissionInfo) {
                transmissionInfo = new Vector<Vector<double[]>>();
                if (transmissionInfoFile != null) {
                    try {
                        transmissionWriter = new BufferedWriter(new FileWriter(transmissionInfoFile));
                    } catch (IOException ioe) {
                        System.err.println("Exception creating transmission writer.");
                        transmissionInfoFile = null;
                        transmissionWriter = null;
                    }
                }
                cumulativeCutDepthCorrelation = new double[8];
                cutDepthCorrCount = new int[8];
                allCutDepthCorrelations = new Vector<double[]>();
            }
            while (generationNumber < maxGenerations && !stopCriterion.stopReached(this) && !stopnowflag) {
                if (collectTransmissionInfo) {
                    currentTransmissionVector = new Vector<double[]>();
                    transmissionInfo.add(currentTransmissionVector);
                }

                //moved up 150607
                if (stopCriterion instanceof EvaluativeStoppingCriterion && ((EvaluativeStoppingCriterion) stopCriterion).usesValidation()) {
                    setValidationFitnessValues(validationInputs, validationTargetOutputs);
                }
                if (stopCriterion instanceof StagnationStoppingCriterion && ((StagnationStoppingCriterion) stopCriterion).lastGenerationWasImproved()) {
                    savePopulation();
                }

                evolveOneGeneration(steadyState, selectMethod, inputs, targetOutputs);
                //if(collectTransmissionInfo)
                //	transmissionInfo.add(currentTransmissionVector);

                // Added by SGO
                setValidationFitnessValues(validationInputs, validationTargetOutputs);

                this.fireGen();

                if (printInfo) {
                    System.out.println("pop size = " + popSize);
                }

                //if(stopCriterion instanceof EvaluativeStoppingCriterion && ((EvaluativeStoppingCriterion)stopCriterion).usesValidation())
                //	setValidationFitnessValues(validationInputs,validationTargetOutputs);
                //if(stopCriterion instanceof StagnationStoppingCriterion && ((StagnationStoppingCriterion)stopCriterion).lastGenerationWasImproved())
                //	savePopulation();

                if (stopCriterion.stopIfNoDiversity() && hasNoDiversity()) {
                    if (transmissionWriter != null) {
                        try {
                            transmissionWriter.flush();
                            transmissionWriter.close();
                        } catch (IOException ioe) {
                            ioe.printStackTrace();
                        }
                    }
                    //for(int i=0;i<popSize;i++)
                    //	System.err.println(individuals[i].toString());
                    this.fireGPDoneEvent();
                    return true;
                }
            }
            if (stopCriterion instanceof StagnationStoppingCriterion) {
                individuals = savedPopulation;
                allFitnesses = savedFitnesses;
                setSummaryFitnessValues();
            }
            setValidationFitnessValues(validationInputs, validationTargetOutputs);
        } catch (InvalidInputSizeException iise) {
            iise.printStackTrace();
            if (transmissionWriter != null) {
                try {
                    transmissionWriter.flush();
                    transmissionWriter.close();
                } catch (IOException ioe) {
                    ioe.printStackTrace();
                }
            }
            this.fireGPDoneEvent();
            return false;
        }
        if (transmissionWriter != null) {
            try {
                transmissionWriter.flush();
                transmissionWriter.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
        this.fireGPDoneEvent();
        return true;
    }
}