package geneticWedge.gp;

/**A StoppingCriterion determines when evolution will stop. Differnet StoppingCriteria may perform evolution for
a fixed number of generations or implement some form of early stopping.*/
public abstract class StoppingCriterion
{
	private boolean stopOnZeroDiversity=false;

	/**If true, evolution will stop prematurely once there is no diversity in the Population (all Individuals are identical).
	By default this parameter is false.*/
	public void stopIfNoDiversity(boolean stop)
	{
		stopOnZeroDiversity=stop;
	}

	/**Returns stopOnZeroDiversity. If true, evolution will stop prematurely once there is no diversity in the
	Population (all Individuals are identical). By default this parameter is false.*/
	public boolean stopIfNoDiversity()
	{
		return stopOnZeroDiversity;
	}

	/**Absract method that must be implmented by non-absract subclasses. Should return true if the stopping condition has been met.*/
	public abstract boolean stopReached(AbstractPopulation pop);

	/**Returns a String represnetation of this StoppingCriterion.*/
	public abstract String toString();
}