package gp.dataAnalysis;

import java.util.*;

public class KMeans
{
	private static Random rand = new Random();
	private static double[][] centroids = null;

	public static void main(String args[])
	{
		int noClusters = 3;
		double[][] points = new double[100][2];
		for(int i=0;i<100;i++)
			for(int j=0;j<2;j++)
				points[i][j] = rand.nextDouble();
		Vector<Integer>[] clusters = clusterData(points,noClusters);
		for(int i=0;i<noClusters;i++)
		{
			System.out.println("Cluster:"+(i+1));
			for(int j=0;j<clusters[i].size();j++)
			{
				int index = clusters[i].get(j);
				String line = points[index][0]+","+points[index][1];
				System.out.println(line);
			}
		}
	}

	public static double[][] getLastCentroids()
	{
		return (double[][])(centroids.clone());
	}

	public static Vector<Integer>[] clusterData(double[][] points, int noClusters)
	{
		Vector<Integer>[] classification = new Vector[noClusters];
		for(int i=0;i<noClusters;i++)
			classification[i] = new Vector<Integer>();
		//seed with random classification
		for(int i=0;i<points.length;i++)
		{
			int cluster = (int)(rand.nextDouble()*noClusters);
			classification[cluster].add(i);
		}

		Vector<Integer>[] newClassification = LloydIteration(points,classification);
		while(!areEqual(newClassification, classification))
		{
			classification = newClassification;
			newClassification = LloydIteration(points,classification);
		}
		return classification;
	}

	private static Vector<Integer>[] LloydIteration(double[][] points, Vector<Integer>[] classification)
	{
		int dimensionality = points[0].length;
		int noClasses = classification.length;
		centroids = new double[noClasses][dimensionality];
		for(int i=0;i<noClasses;i++)
		{
			int classSize = classification[i].size();
			for(int j=0;j<classSize;j++)
			{
				double[] point = points[classification[i].get(j)];
				for(int k=0;k<dimensionality;k++)
					centroids[i][k] += point[k];
			}
			for(int j=0;j<dimensionality;j++)
				centroids[i][j] /= classSize;
		}

		Vector<Integer>[] newClassification = new Vector[noClasses];
		for(int i=0;i<noClasses;i++)
		{
			newClassification[i] = new Vector<Integer>();
		}
		for(int i=0;i<points.length;i++)
		{
			int index = -1;
			double minDistance = Double.MAX_VALUE;
			for(int j=0;j<noClasses;j++)
			{
				double dist = 0.0;
				for(int k=0;k<dimensionality;k++)
				{
					dist += Math.pow(points[i][k]-centroids[j][k],2.0);
				}
				if(dist<minDistance)
				{
					minDistance = dist;
					index = j;
				}
			}
			newClassification[index].add(i);
		}
		return newClassification;
	}

	private static boolean areEqual(Vector<Integer>[] classification1, Vector<Integer>[] classification2)
	{
		int len = classification1.length;
		if(classification2.length != len)
		{
			System.err.println("classification arrays are unequal length");
			return false;
		}
		for(int i=0;i<len;i++)
		{
			if(!classification1[i].equals(classification2[i]))
				return false;
		}
		return true;
	}
}