package geneticWedge.gp.display;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import geneticWedge.gp.*;
import geneticWedge.gp.function.*;

/**TreePanel is a JPanel that draws GP trees onto itself.*/
public class TreePanel extends JPanel
{
	private Vector<geneticWedge.gp.Component>nodes=null;
	int treeDepth;
	private int diameter=40;
	private double fractionalGap=0.15;
	private Dimension panelSize = new Dimension(0,0);

	/**Constructor takes the Individual that it is to draw as an input*/
	public TreePanel(Individual ind)
	{
		update(ind);
	}

	/**Changes the Individual to be drawn*/
	public void update(Individual ind)
	{
		nodes=ind.getNodes();
		treeDepth=ind.getDepth();
		setPanelSize();
	}

	public void paintComponent(Graphics g)
	{
		Graphics2D g2=(Graphics2D)g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setStroke(new BasicStroke(2));
		g2.setFont(new Font("Arial",Font.BOLD,20));
		FontMetrics fm = g2.getFontMetrics();


		int currentDepth=0;
		Vector<Point> branchPoints=new Vector<Point>();
		Vector<Integer> branchDepths=new Vector<Integer>();

		int currentX=getWidth()/2;
		int currentY=diameter;
		for(int i=0;i<nodes.size();i++)
		{
			geneticWedge.gp.Component currentNode=nodes.get(i);
			Point currentPoint = new Point(currentX,currentY);

			if(currentNode instanceof Function)
				g2.setColor(Color.RED);
			else if(currentNode instanceof Input)
				g2.setColor(Color.GREEN);
			else
				g2.setColor(Color.YELLOW);

			drawNode(g2, fm, currentNode, currentPoint);

			if(currentNode instanceof TwoInputFunction)
			{
				int xSpacing = getXSpacing(treeDepth-currentDepth-1, diameter);
				branchPoints.add(currentPoint);
				branchDepths.add(currentDepth);
				g2.drawLine(currentX,currentY+diameter/2,currentX+xSpacing,currentY+diameter*3/2);
				g2.drawLine(currentX,currentY+diameter/2,currentX-xSpacing,currentY+diameter*3/2);
				currentX -= xSpacing;
				currentY += diameter*2;
				currentDepth++;
			}
			else if(currentNode instanceof OneInputFunction)
			{
				g2.drawLine(currentX,currentY+diameter/2,currentX,currentY+diameter*3/2);
				currentY += diameter*2;
				currentDepth++;
			}
			else
			{
				int index=branchDepths.size();
				if(index>0)
				{
					currentDepth=branchDepths.get(index-1)+1;
					branchDepths.remove(index-1);
					Point pt=branchPoints.get(index-1);
					currentY=pt.y + diameter*2;
					currentX=pt.x + getXSpacing(treeDepth-currentDepth, diameter);
					branchPoints.remove(index-1);
				}
			}
		}
	}

	private void setPanelSize()
	{
		int wid = 0;
		for(int i=0;i<treeDepth;i++)
			wid+=getXSpacing(i,diameter);
		wid = (wid+diameter)*2;
		int ht = (treeDepth*2+1)*diameter;
		setSize(wid,ht);
		panelSize = new Dimension(wid,ht);
	}

	public Dimension getPanelSize()
	{
		return panelSize;
	}
	private void drawNode(Graphics2D g2, FontMetrics fm, geneticWedge.gp.Component node, Point pt)
	{
		g2.fillOval(pt.x-diameter/2, pt.y-diameter/2, diameter, diameter);
		g2.setColor(Color.BLACK);
		g2.drawOval(pt.x-diameter/2, pt.y-diameter/2, diameter, diameter);
		String nodeStr=node.toString();
		Rectangle2D nodeRect=fm.getStringBounds(nodeStr,g2);
		double ht=fm.getLineMetrics(nodeStr,g2).getAscent();
		double wid=nodeRect.getWidth();
		g2.drawString(nodeStr,(int)(pt.x-wid/2.0), (int)(pt.y+ht/2.0));
	}

	private int getXSpacing(int height, int diameter)
	{
		double space=0.0;
		double cumulativeSpace=0.0;
		for(int i=0;i<height;i++)
		{
			space=space+diameter*(0.5+fractionalGap);
			if(cumulativeSpace+diameter*(0.5+fractionalGap)>space)
				space=cumulativeSpace+diameter*(0.5+fractionalGap);
			cumulativeSpace+=space;
		}
		return (int)space;
	}
}
