package geneticWedge.gp.function;

import geneticWedge.gp.*;

/**BipolarSigmoidFunction is a unary function commonly used in Artificial Neural Networks.*/
public class BipolarSigmoidFunction extends OneInputFunction
{
	/**Constructor*/
	public BipolarSigmoidFunction()
	{
		super();
		complexity = 2;
	}

	/**getOutput() throws InvalidInputSizeException if the length of the input array is not 1.*/
	public double getOutput(double[] input) throws InvalidInputSizeException
	{
		if(input.length!=1)
			throw new InvalidInputSizeException();
		else
		{
			double exp=Math.exp(input[0]);
			double out=(1.0-exp)/(1.0+exp);
			if(Double.isInfinite(out)||Double.isNaN(out))
				return 0.0;
			else
				return out;
		}
	}

	/**Returns a String representation of an BipolarSigmoidFunction object ('BiSig').*/
	public String toString()
	{
		return "BiSig";
	}
}