package geneticWedge.gp.function;

import geneticWedge.gp.*;

/**The abstract parent of all Functions. Functions are Components that are used within a tree, i.e. not at its leaves.
They must be able to calculate an output given an array of inputs. Generally the input array must be of a specific length
otherwise an InvalidInputSizeException will be thrown upon calling getOutput().*/
public abstract class Function extends Component
{
	/*protected*/ int noOfInputs;
	/*protected*/ int complexity;

	/**All non-abstract subclasses must implement this method, so that they can transform inputs into a single output.*/
	public abstract double getOutput(double[] inputs) throws InvalidInputSizeException;

	/**Returns a String representation of this Function object. The String representations should be different for different classes.*/
	public abstract String toString();

	/**Should return the length of the array required by getOutput()*/
	public int getNoOfInputs()
	{
		return noOfInputs;
	}

	/**Should return the complexity of the Function. Simple arithmetic functions have complexity 0, while
	higher order functions like <i>pow</i> have higher complexities.*/
	public int getComplexity()
	{
		return complexity;
	}
}