package geneticWedge.gp.function;

import geneticWedge.gp.*;

/**Abstract parent of all binary functions. Subclasses should throw an InvalidInputSizeException if the length of the array
sent to getOutput() is not 2.*/
public abstract class TwoInputFunction extends Function
{
	/**The constructor sets the number of inputs required to 2*/
	public TwoInputFunction()
	{
		noOfInputs=2;
	}

	/**This constructor allows the creation of Functions with assigned complexity values, rather than the default values.
	This may be used to give priority to chosen functions (by giving them a low complexity value) in conjunction with
	the use of the FUNCTION_COMPLEXITY PerformanceFunction in Population.*/
	public TwoInputFunction(int complexity)
	{
		this();
		this.complexity = complexity;
	}

	/**non-abstract subclasses must implement this method. The isCommutative method is used to decide whether a
	node is an appropriate location for performing SWAP mutation: if a Function is commutative SWAP will not be applied to it.*/
	public abstract boolean isCommutative();
}