package geneticWedge.gp.function;

import geneticWedge.gp.*;

/**UnpolarSigmoidFunction is a unary function commonly used in Artificial Neural Networks.
It is also known as a 'logistic' function.*/
public class UnipolarSigmoidFunction extends OneInputFunction
{
	/**constructor*/
	public UnipolarSigmoidFunction()
	{
		super();
		complexity = 2;
	}

	/**getOutput() throws InvalidInputSizeException if the length of the input array is not 1.*/
	public double getOutput(double[] input) throws InvalidInputSizeException
	{
		if(input.length!=1)
			throw new InvalidInputSizeException();
		else
		{
			double out=1.0/(1.0+Math.exp(input[0]));
			if(Double.isInfinite(out)||Double.isNaN(out))
				return 0.0;
			else
				return out;
		}
	}

	/**Returns a String representation of an UnipolarSigmoidFunction object ('uniSig').*/
	public String toString()
	{
		return "uniSig";
	}
}