package geneticWedge.gp.io;

import java.io.*;
import java.util.Vector;

/**This class contains static methods for reading csv (comma-separated value) files and returning StringData or NumericData objects.*/
public class CSVReader
{
	/**Returns a NumericData object by reading a csv file named <i>filename</i>. If the first row of the csv file contains variable names,
	this will be automatically detected by the reader and the variable names may be retrieved from the NumericData object by calling getNames().
	@param filename A path to the csv file
	@param labelsInFirstColumn If true, the first column will be treated as String labels rather than data.
	The labels will be saved and can be retrieved from the NumericData object by calling getLabels()*/
	public static NumericData readFile(String filename, boolean labelsInFirstColumn) throws FileNotFoundException, IOException, NumberFormatException
	{
		return TextReader.readFile(filename,",",labelsInFirstColumn);
	}

	/**Returns a StringData object by reading a csv file named <i>filename</i>.
	@param filename A path to the csv file
	@param namesInFirstRow If true, the first column will be treated as variable names rather than data.
	The names will be saved and can be retrieved from the NumericData object by calling getNames()
	@param labelsInFirstColumn If true, the first column will be treated as String labels rather than data.
	The labels will be saved and can be retrieved from the NumericData object by calling getLabels()*/
	public static StringData readStringFile(String filename, boolean namesInFirstRow, boolean labelsInFirstColumn) throws FileNotFoundException, IOException
	{
		return TextReader.readStringFile(filename,",", namesInFirstRow, labelsInFirstColumn);
	}

	/**Removes duplicated data from the dataset in <i>filename<i> and writes the 'clean' dataset to <i>newFileName</i>.*/
	public static void removeDuplicates(String filename, String newFileName, boolean labelsInFirstColumn) throws IOException
	{
		TextReader.removeDuplicates(filename, newFileName, labelsInFirstColumn, ",");
	}
}