package geneticWedge.gp.io;

import java.sql.*;
import java.util.Vector;

/**This class contains static methods for reading data from Excel files and returning NumericData objects.
The Excel file to be read should have been registered with ODBC.*/
public class ExcelReader
{
	/**This method reads all of the data in <i>worksheet<i>, within <i>spreadsheet<i>. If <i>labelsInFirstColumn<i> is true,
	The contents of the first column will be used as data labels. They may be retrieved from the NumericData
	object by calling getLabels(). If variable names are in the first row of the spreadsheet, this will be automatically detected.
	The variable names may be retrieved from the NumericData object by calling getNames().*/
    public static NumericData readWorksheet(String spreadsheet, String worksheet, boolean labelsInFirstColumn)
    {
        Connection con = null;
        Statement stmnt = null;
        String[] labels = null;
        Vector<String> labelVector = new Vector<String>();
        try
        {
            Class.forName( "sun.jdbc.odbc.JdbcOdbcDriver" );
            con = DriverManager.getConnection( "jdbc:odbc:"+spreadsheet, "", "" );
            stmnt = con.createStatement();
            String query = "select * from ["+worksheet+"$];";
            ResultSet rs = stmnt.executeQuery( query );
            ResultSetMetaData meta = rs.getMetaData();
            int noCols = meta.getColumnCount();
            if(labelsInFirstColumn)
            	noCols--;
            String[]names = new String[noCols];
            for(int i=1;i<=noCols;i++)
            {
				if(labelsInFirstColumn)
					names[i-1]=meta.getColumnName(i+1);
				else
            		names[i-1]=meta.getColumnName(i);
			}

            Vector vectorData = new Vector<double[]>();

            int count=0;
            while( rs.next() )
            {
				double[] row=new double[noCols];
				if(labelsInFirstColumn)
				{
					labelVector.add(rs.getString(1));
					for(int i=1;i<=noCols;i++)
						row[i-1]=rs.getDouble(i+1);
				}
				else
				{
					for(int i=1;i<=noCols;i++)
						row[i-1]=rs.getDouble(i);
				}
                vectorData.add(row);
                count++;
            }
            double[][] data=new double[count][noCols];
            vectorData.toArray(data);
            if(labelsInFirstColumn)
            {
				labels = new String[labelVector.size()];
				for(int i=0;i<labelVector.size();i++)
					labels[i] = labelVector.get(i);
			}
            return new NumericData(names, data, labels);
        }
        catch( Exception e )
        {
            System.err.println( e );
            return null;
        }
        finally
        {
            try
            {
                stmnt.close();
                con.close();
            }
            catch( Exception e )
            {
                System.err.println( e );
            }
        }
    }
}