package geneticWedge.gp.io;

import java.util.Vector;
import java.util.Hashtable;

/**This class can extract String data from files and manipulate it in various ways.*/
public class StringData
{
	private String[] names;
	private String[][] data;
	private String[] labels;
	private int dimensionality;
	private int dataLength;

	/**Constructor takes variable names, data and data labels.*/
	public StringData(String[] names, String[][]data, String[] labels)
	{
		this.names=names;
		this.data=data;
		this.labels = labels;
		dataLength=data.length;
		dimensionality=data[0].length;
	}

	/**Returns the data held by this StringData object.*/
	public String[][] getData()
	{
		return data;
	}

	/**Returns the variable names of the data.*/
	public String[] getNames()
	{
		return names;
	}

	/**Returns the data labels of the data.*/
	public String[] getLabels()
	{
		return labels;
	}

	/**Converts the String data in selected columns into numeric data.
	A NumberFormatException will be thrown if any of the selected columns do not contain numeric data.*/
	public NumericData toNumericData(int[] numericColumns) throws NumberFormatException
	{
		int noNumeric = numericColumns.length;

		double[][] numericData = new double[dataLength][noNumeric];
		for(int i=0;i<dataLength;i++)
		{
			for(int j=0;j<noNumeric;j++)
			{
				numericData[i][j] = Double.parseDouble(data[i][numericColumns[j]]);
			}
		}
		String[] newNames = null;
		if(names!=null)
		{
			newNames = new String[noNumeric];
			for(int i=0;i<noNumeric;i++)
				newNames[i] = names[numericColumns[i]];
		}
		System.err.println("Returning data");
		return new NumericData(newNames, numericData, labels);
	}

	/**Converts the String data in selected columns into numeric data.
	The Hashtable contains conversions between String representations and numeric values, so that non-numeric Strings may be converted to numeric values.*/
	public NumericData toNumericData(int[] numericColumns, Hashtable<String,Double> hash)
	{
		int noNumeric = numericColumns.length;
		double[][] numericData = new double[dataLength][noNumeric];
		for(int i=0;i<dataLength;i++)
		{
			for(int j=0;j<noNumeric;j++)
			{
				numericData[i][j] = hash.get(data[i][numericColumns[j]]);
			}
		}
		String[] newNames = null;
		if(names!=null)
		{
			newNames = new String[noNumeric];
			for(int i=0;i<noNumeric;i++)
				newNames[i] = names[numericColumns[i]];
		}
		return new NumericData(newNames, numericData, labels);
	}

	/**This method checks which columns of the data contain numeric data.*/
	public int[] getNumericIndices()
	{
		Vector<Integer> indices = new Vector<Integer>();
		for(int i=0;i<dimensionality;i++)
		{
			try
			{
				Double.parseDouble(data[1][i]);//use second row - first row may contain names
			}
			catch(NumberFormatException nfe)
			{
				continue;
			}
			indices.add(i);
		}
		int len = indices.size();
		int[] indicesArray = new int[len];
		for(int i=0;i<len;i++)
			indicesArray[i] = indices.get(i);
		return indicesArray;
	}

	/**Returns the dimensionality of the data.*/
	public int getDimensionality()
	{
		return dimensionality;
	}

	/**Returns the number of data items in the dataset.*/
	public int getDataLength()
	{
		return dataLength;
	}
}