package geneticWedge.gp.io;

import org.xml.sax.*;
import org.xml.sax.helpers.*;

/**This class contains static methods for reading XML files and returning NumericData objects.*/
public class XMLDataReader
{
	/**Returns a NumericData object by reading an xml file located at <i>uri</i>.
	@param uri A location, either web-based or local
	@param dataNames The names of the variables to be retrieved for use as data
	@param label The name of the variable to be used as a data label
	@param dataMode Determines which data will be retrieved: ELEMENTS_AND_ATTRIBUTES, ELEMENTS_ONLY or ATTRIBUTES_ONLY
	The labels will be saved and can be retrieved from the NumericData object by calling getLabels()*/
	public static NumericData readFile(String uri, String[] dataNames, String label, XMLTreeHandler.IncludeData dataMode)
	{
		XMLTreeHandler handler = null;
		try
		{
			XMLReader reader = XMLReaderFactory.createXMLReader();
			//validation commented out
			/*reader.setFeature("http://xml.org/sax/features/validation",true);
			reader.setFeature("http://apache.org/xml/features/validation/schema",true);*/

			handler = new XMLTreeHandler(uri, dataMode);

			reader.setContentHandler(handler);
			reader.setErrorHandler(handler);

			InputSource inSource = new InputSource(uri);
			reader.parse(inSource);
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.exit(1);
		}

		String[] names = handler.getData(label);
		int dimensionality = dataNames.length;
		double[][] data = new double[dimensionality][];
		for(int i=0;i<dimensionality;i++)
		{
			String[] stringData = handler.getData(dataNames[i]);
			if(stringData==null)
			{
				data[i]=null;
				continue;
			}
			int dataLength = stringData.length;
			data[i] = new double[dataLength];
			for(int j=0;j<dataLength;j++)
			{
				try
				{
					data[i][j] = Double.parseDouble(stringData[j]);
				}
				catch(NumberFormatException nfe)
				{
					data[i][j] = Double.NaN;
				}

			}
		}
		return new NumericData(names,data,dataNames);
	}
}