package geneticWedge.gp.io;

import geneticWedge.gp.*;
import geneticWedge.gp.function.*;
import java.io.*;
import java.util.Vector;

/**This class allows the writing of an Individual as an XML file.*/
public class XMLWriter
{
	/**This method writes the nodes within an Individual as an XML file.*/
	public static void writeXML(Individual ind, String filename)
	{
		try
		{
			BufferedWriter writer = new BufferedWriter(new FileWriter(new File(filename)));
			writer.write("<?xml version=\"1.0\"?>");
			writer.newLine();
			Vector<Component> nodes = ind.getNodes();

			Vector<Component> openNodes=new Vector<Component>();

			Vector<Boolean> firstTerminalFound = new Vector<Boolean>();
			int len = nodes.size();
			for(int nodeCount=0;nodeCount<len;nodeCount++)
			{
				Component comp=nodes.get(nodeCount);
				if (comp instanceof Terminal)
				{
					if(comp instanceof Constant)
						writer.write("<"+comp.getClass().getSimpleName()+" value=\""+comp+"\"/>");
					else
						writer.write("<"+comp.getClass().getSimpleName()+" name=\""+comp+"\"/>");
					writer.newLine();
					if(openNodes.size()>0 && (openNodes.lastElement() instanceof OneInputFunction))
						closeNodes(writer, openNodes, firstTerminalFound);
					else if (firstTerminalFound.size()>0 && firstTerminalFound.lastElement())
						closeNodes(writer, openNodes, firstTerminalFound);
					else if(firstTerminalFound.size()>0)
						firstTerminalFound.set(firstTerminalFound.size()-1, true);
				}
				else
				{
					openNodes.add(comp);
					firstTerminalFound.add(false);
					writer.write("<"+comp.getClass().getSimpleName()+" symbol=\""+comp+"\">");
					writer.newLine();
				}
			}

			writer.flush();
			writer.close();
		}
		catch (IOException ex)
		{
		  ex.printStackTrace();
		}
	}

	private static void closeNodes(BufferedWriter writer, Vector<Component> openNodes, Vector<Boolean> firstTerminalFound) throws IOException
	{
		writer.write("</"+openNodes.lastElement().getClass().getSimpleName()+">");
		writer.newLine();
		openNodes.removeElementAt(openNodes.size()-1);
		firstTerminalFound.removeElementAt(firstTerminalFound.size()-1);

		while(openNodes.size()>0 && (openNodes.lastElement() instanceof OneInputFunction || firstTerminalFound.lastElement()))
		{
			writer.write("</"+openNodes.lastElement().getClass().getSimpleName()+">");
			writer.newLine();
			openNodes.removeElementAt(openNodes.size()-1);
			firstTerminalFound.removeElementAt(firstTerminalFound.size()-1);
		}
		if(openNodes.size()>0)
			firstTerminalFound.set(firstTerminalFound.size()-1, true);
	}
}